


//Program        : Java 1.5 Examples

//Author         : Richard E. Pattis
//Computer Science Department
//Carnegie Mellon University
//5000 Forbes Avenue
//Pittsburgh, PA 15213-3891
//e-mail: pattis@cs.cmu.edu

//Maintainer     : Author


//Description:

//This program illustrates the use of a variety of features new to Java 1.5.
//These features are often highly interrelated, and they all work together
//synergistically. Each feature can be examined apart from the others, but
//often the examples below illustrate multiple features. Here are the basic
//features illustrated in this program

//0. Static Imports
//1. Extended for loops for arrays
//2. Autoboxing and auto-unboxing
//3. Methods with a variable number of arguments
//4. C-stlye String formatting (for I/O)
//5. Using generics for classes (and methods)
//6. Extended for loops for generics

//Known Bugs     : None

//Future Plans   : None

//Program History:
//8/17/06: R. Pattis - Operational in Java





import edu.cmu.cs.pattis.cs151xx.Prompt;
import java.util.*;

//See 0 below (allows omitting class/variable name when accessing its members)
import static java.lang.Math.*;       //Allow static methods in Math minus Math prefix
import static java.lang.System.*;   //Allow out.print/out.println minus System prefix



public class Application {

	//See import static above
	public static void demoImportStatic(double x) {
		double f = log(abs(x));   //Note, not Math.log(Math.abs(x));
		out.println("f = " + f);  //Note, not System.out.println(...);
	}

	//See 2 below
	//Note: changing either Integer to int has no effect
	public static Integer factorial (Integer i) {
		return (i==0 ? 1 : i*factorial(i-1));
	}



	//See 3 below
	//int... means int[], with the array built automatically
	//  from all the optonal arguments following the first
	//  (of course, they all must be ints)
	//Note use of the extended for loop (see 1 below)
	//We could write this metthod using Integer... and Java
	//  would autobox it for us
	public static int max (int x1, int... xs) {
		int answer = x1;
		for (int x : xs)
			if (answer < x)
				answer = x;
		return answer;
	}

	//See 2 below
	//Autoboxing used for primitive arguments
	//Note use of the extended for loop (see 1 below)
	public static String catenator (Object...os) {
		String answer = "";
		for (Object o : os)
			answer += o;
		return answer;
	}


	public static void main(String[] args) {
		//0. Static Imports
		demoImportStatic(-2.);


		//1. Extended for loops for arrays
		//If T[] a = ...allocation
		//for (T x : a)      for(int i=0; i<a.length; i++) {
		//  s;                  T x = a[i];
		//                      s;
		//                   }
		//(see also max/catentaor methods above main)
		boolean[] prime = {false, true, true, false, true, false};
		System.out.print("prime=");   
		for (boolean p : prime)
			System.out.print(" "+p);
		System.out.println();



		//2. Autoboxing and auto-unboxing
		//The Java compiler will automatically generate code to convert between
		//  int and Integer (the wrapper class for int) epresentations of integer
		//  values; likewise for other primitive types and their wrapper classes.
		//Method resolution (1) regular; (2) 1 + autoboxing; (3) 2+ var args)
		//Note: Integer i1 = 1, i2 = 2; if (i1 == i2) ... doesn't do .equals check!
		System.out.println("boxing/unboxing");
		Integer i = 0;  //Integer i = new Integer(0);

		System.out.println("  i is now " + i);
		i = i + 1;      //i = new Integer(i.intValue() + 1);
		i++;            //same, again
		i+=2;           //i = new Integer(i.intValue() + 2);

		//See factorial method before main
		System.out.println("  "+i+"! = " + factorial(i));

		//Same for other type wrappers
		Boolean b = false;
		System.out.println("  !"+b+" = " + !b);

		//Works in arrays too
		Double[] doubleArray = {0., .5, 1.0, 1.5, 2.0};
		System.out.print("  doubleArray = ");
		for (double d : doubleArray)
			System.out.print(d + "  ");  //Note 5 (extended for loop)
		System.out.println();

		//What happens when an Integer is null, and we unbox it?
		i = null;
		try {
			int temp = i;
			System.out.println(temp);
		} catch (Exception e)
		{System.out.println("  Exception thrown: " + e.getClass());}



		//3. Methods with a variables number of arguments (see methods before main)
		//After "required" arguments, puts multiple "optional" arguments into an
		//  array (and passes the array)
		System.out.println("\nVar-args");
		System.out.println("  max(1)     = " + max(1));
		System.out.println("  max(1,2)   = " + max(1,2));
		System.out.println("  max(1,3,2) = " + max(1,3,2));
		System.out.println("  max(1,3,2) = " + max(1,3,2));
		System.out.println("  max(1,3,2) = " + max(1,new int[]{3,2}));

		System.out.println("  catenator()             = " + catenator());
		System.out.println("  catenator(\"A\",\"BC\",\"D\") = " + catenator("A","BC","D%30F"));
		System.out.println("  catenator(\"A\",1,true)   = " + catenator("A",1,true));



		//4. C-stlye String formatting (for I/O)
		//The form is %[index$][flags][width][.precision]conversion
		//  index: argument number; if discarded "next" argument
		//  flags: - left-justification (right padding) of width
		//         + numeric value should include +/- sign
		//         # special form for octal/hexadecimal numbers (with %0, %x)
		//         0 pad with 0 (not spaces)
		//         ( negative numbers should appear in parentheses
		//  width: length of resulting string; if discarded, no padding
		//  .precision: decimal places for non-integral numbers
		//  conversion: d   - decimal
		//              e/E - floating point with exponential
		//              f   - floating point without exponential
		//              s/S - String (via toString)
		//              b/B - boolean (i.e., true/TRUE)
		//              o   - octal
		//              h/H - hexadecimal
		//              t/T - time formats (followed by another letter for details)
		//              others as well
		//Throw exception if bad format string or not enough arguments
		//  if too many arguments, just ignores them
		//Also String.format(Format, args...) produces String,
		//  so System.out.printf(F,a...); is the same as
		//     System.out.prtint( String.format(F,a...) );
		System.out.println("\nC-style formatting");
		System.out.printf("  |%d|%10d|%-10d|%#X|\n", 10,20,30,40);

		for(;;) {
			String format = Prompt.forString("\n  Enter format for double (STOP to terminate)","STOP");
			if (format.equals("STOP"))
				break;
			double value  = Prompt.forDouble("  Enter double value");
			try {
				System.out.printf("  "+format+"\n",value);	
			} catch (Exception e)
			{System.out.println("    Exception thrown: " + e.getClass()+" : "+e.getMessage());}
		}

		
		
		//5. Using generics for classes (and methods)
		//  We will see much more on this information later
		//6. Extended for loops	for generics
		List<String> names = new ArrayList<String>();
		names.add("John"); names.add("Paul"); names.add("George"); names.add("Ringo");

		Set<Integer> ages = new HashSet<Integer>();
		ages.add(31); ages.add(32); ages.add(33); ages.add(34); //Autoboxing!

		System.out.print("\nUsing generics");
		System.out.print("\n  Names:");
		for (String name : names)
			System.out.print(" "+name);
		System.out.print("\n  Ages :");
		for (Integer age : ages)
			System.out.print(" "+age);
	}
}
